#ifndef EVENT_H
#define EVENT_H
#include "config.h"
#include "obj_struct.h"
#include "coordinate.h"

typedef enum {
    et_invalid,
    et_begin,
    et_start = et_begin,     // ミッション開始

    et_complete,   // ミッション完了
    et_fail,      // ミッション失敗(自機撃墜以外)
    et_die,        // 自機撃墜
    et_in,          // 指定エリアに入った
    et_out,         // 指定エリアを出た
    et_time,        // 指定時刻になった
    et_end,
} event_trigger_type_t;

typedef enum {
    eo_invalid,
    eo_begin,
    eo_create = eo_begin,          // オブジェクトを生成
    eo_create_region,   // オブジェクト群をランダム配置した領域を生成
    eo_delete,   // オブジェクトを削除(空間から単純に除去)
    eo_destroy,  // オブジェクトを破壊(ダメージによる破壊と同じ扱い)
    eo_end,
} event_op_type_t;

typedef int event_id_t;
#define INVALID_EVENT_ID (-1)

void event_init(void);
void reset_event_list(void);
event_trigger_type_t event_trigger_name2type(char *str);
event_op_type_t event_op_name2type(char *str);
event_id_t enqueue_event_create(event_trigger_type_t et,
                                obj_type_t obj_type, pos3_t pos3);
event_id_t enqueue_event_create_region(event_trigger_type_t et,
                                       obj_type_t obj_type, region3_t region3,
                                       int num);
void register_events(void);
#ifdef DUMMY
// event構造体の中身はevent.c外に公開しない
typedef struct event_s *Event_t;
typedef struct event_list_s *EventList_t;

EventList_t create_event_list(void);
Event_t create_event(EventList_t event_list);
void set_event_create(Event_t event, obj_type_t obj_type, pos3_t pos3);
void set_event_create_region(Event_t event, obj_type_t obj_type, 
                             region3_t region3, int num);
#endif

#endif

