#ifndef PROCESS_XPM_H
#define PROCESS_XPM_H

// 扱える色数
#define XPM_COLOR_NUM_MAX 256
// 色名文字列の長さ上限
#define XPM_COLOR_NAME_LEN_MAX 128
// 1ピクセルに使うバイト数上限
#define DEPTH_MAX 2

// 1ピクセルの色を表す文字(2文字使うこともあるがnullターミネートしない)
typedef struct {
    char chr[2];
} pixel_str_t;

// 一色分の情報
typedef struct {
    pixel_str_t str; // パターン中でその色に使う文字(1,2文字)
    char name[XPM_COLOR_NAME_LEN_MAX]; // 色名。RGB形式も含めXPMで書ける文字列
} xpm_color_t;

// XPM(char*[])を操作用に変換した形式
typedef struct {
    int width_max, height_max; // 格納可能な上限サイズ(確保領域の大きさ)
    int width, height, color_num, depth;
    pixel_str_t transparent_str;
    xpm_color_t colors[XPM_COLOR_NUM_MAX];
    pixel_str_t *pixels;
} dynamic_xpm_t;

dynamic_xpm_t *alloc_dynamic_xpm(int width, int height);
void free_dynamic_xpm(dynamic_xpm_t *dxpm);
char **alloc_xpm_buffer(int width, int height, int color_num);
void free_xpm_buffer(char **xpm);
int xpm2dynamic_xpm(char **xpm, dynamic_xpm_t *dxpm);
int dynamic_xpm2xpm(dynamic_xpm_t *dxpm, char **xpm);
int overlay_xpm(dynamic_xpm_t *lower_dxpm, dynamic_xpm_t *upper_dxpm, 
                int offset_x, int offset_y,
                dynamic_xpm_t *result_dxpm);
int magnify_xpm_int(int xscale, int yscale,
                    dynamic_xpm_t *src_dxpm, dynamic_xpm_t *dest_dxpm);
int magnify_xpm(double xscale, double yscale,
                dynamic_xpm_t *src_dxpm, dynamic_xpm_t *dest_dxpm);
int save_xpm(char *path, char *name, char **xpm);

#ifdef DEBUG
void show_dynamic_xpm(dynamic_xpm_t *dxpm);
int show_xpm(char **xpm);
#endif

#endif
